@@ -50,7 +50,7 @@ Revision history:
 /* syslog module */
 
 #include "Python.h"
-#include "osdefs.h"
+#include "osdefs.h"               // SEP
 
 #include <syslog.h>
 
@@ -99,7 +99,7 @@ syslog_get_argv(void)
     if (slash == -2)
         return NULL;
     if (slash != -1) {
-        return PyUnicode_Substring(scriptobj, slash, scriptlen);
+        return PyUnicode_Substring(scriptobj, slash + 1, scriptlen);
     } else {
         Py_INCREF(scriptobj);
         return(scriptobj);
@@ -116,7 +116,7 @@ syslog_openlog(PyObject * self, PyObject * args, PyObject *kwds)
     long facility = LOG_USER;
     PyObject *new_S_ident_o = NULL;
     static char *keywords[] = {"ident", "logoption", "facility", 0};
-    char *ident = NULL;
+    const char *ident = NULL;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds,
                           "|Ull:openlog", keywords, &new_S_ident_o, &logopt, &facility))
@@ -139,16 +139,19 @@ syslog_openlog(PyObject * self, PyObject * args, PyObject *kwds)
      * If NULL, just let openlog figure it out (probably using C argv[0]).
      */
     if (S_ident_o) {
-        ident = _PyUnicode_AsString(S_ident_o);
+        ident = PyUnicode_AsUTF8(S_ident_o);
         if (ident == NULL)
             return NULL;
     }
 
+    if (PySys_Audit("syslog.openlog", "sll", ident, logopt, facility) < 0) {
+        return NULL;
+    }
+
     openlog(ident, logopt, facility);
     S_log_open = 1;
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 
@@ -167,10 +170,14 @@ syslog_syslog(PyObject * self, PyObject * args)
             return NULL;
     }
 
-    message = _PyUnicode_AsString(message_object);
+    message = PyUnicode_AsUTF8(message_object);
     if (message == NULL)
         return NULL;
 
+    if (PySys_Audit("syslog.syslog", "is", priority, message) < 0) {
+        return NULL;
+    }
+
     /*  if log is not opened, open it now  */
     if (!S_log_open) {
         PyObject *openargs;
@@ -195,13 +202,15 @@ syslog_syslog(PyObject * self, PyObject * args)
 static PyObject *
 syslog_closelog(PyObject *self, PyObject *unused)
 {
+    if (PySys_Audit("syslog.closelog", NULL) < 0) {
+        return NULL;
+    }
     if (S_log_open) {
         closelog();
         Py_CLEAR(S_ident_o);
         S_log_open = 0;
     }
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static PyObject *
@@ -211,6 +220,9 @@ syslog_setlogmask(PyObject *self, PyObject *args)
 
     if (!PyArg_ParseTuple(args, "l;mask for priority", &maskpri))
         return NULL;
+    if (PySys_Audit("syslog.setlogmask", "(O)", args ? args : Py_None) < 0) {
+        return NULL;
+    }
     omaskpri = setlogmask(maskpri);
     return PyLong_FromLong(omaskpri);
 }
@@ -240,7 +252,7 @@ syslog_log_upto(PyObject *self, PyObject *args)
 /* List of functions defined in the module */
 
 static PyMethodDef syslog_methods[] = {
-    {"openlog",         (PyCFunction) syslog_openlog,           METH_VARARGS | METH_KEYWORDS},
+    {"openlog",         (PyCFunction)(void(*)(void)) syslog_openlog,           METH_VARARGS | METH_KEYWORDS},
     {"closelog",        syslog_closelog,        METH_NOARGS},
     {"syslog",          syslog_syslog,          METH_VARARGS},
     {"setlogmask",      syslog_setlogmask,      METH_VARARGS},
@@ -249,72 +261,55 @@ static PyMethodDef syslog_methods[] = {
     {NULL,              NULL,                   0}
 };
 
-/* Initialization function for the module */
 
-
-static struct PyModuleDef syslogmodule = {
-    PyModuleDef_HEAD_INIT,
-    "syslog",
-    NULL,
-    -1,
-    syslog_methods,
-    NULL,
-    NULL,
-    NULL,
-    NULL
-};
-
-PyMODINIT_FUNC
-PyInit_syslog(void)
+static int
+syslog_exec(PyObject *module)
 {
-    PyObject *m;
-
-    /* Create the module and add the functions */
-    m = PyModule_Create(&syslogmodule);
-    if (m == NULL)
-        return NULL;
-
-    /* Add some symbolic constants to the module */
-
+#define ADD_INT_MACRO(module, macro)                                  \
+    do {                                                              \
+        if (PyModule_AddIntConstant(module, #macro, macro) < 0) {     \
+            return -1;                                                \
+        }                                                             \
+    } while (0)
     /* Priorities */
-    PyModule_AddIntConstant(m, "LOG_EMERG",       LOG_EMERG);
-    PyModule_AddIntConstant(m, "LOG_ALERT",       LOG_ALERT);
-    PyModule_AddIntConstant(m, "LOG_CRIT",        LOG_CRIT);
-    PyModule_AddIntConstant(m, "LOG_ERR",         LOG_ERR);
-    PyModule_AddIntConstant(m, "LOG_WARNING", LOG_WARNING);
-    PyModule_AddIntConstant(m, "LOG_NOTICE",  LOG_NOTICE);
-    PyModule_AddIntConstant(m, "LOG_INFO",        LOG_INFO);
-    PyModule_AddIntConstant(m, "LOG_DEBUG",       LOG_DEBUG);
+    ADD_INT_MACRO(module, LOG_EMERG);
+    ADD_INT_MACRO(module, LOG_ALERT);
+    ADD_INT_MACRO(module, LOG_CRIT);
+    ADD_INT_MACRO(module, LOG_ERR);
+    ADD_INT_MACRO(module, LOG_WARNING);
+    ADD_INT_MACRO(module, LOG_NOTICE);
+    ADD_INT_MACRO(module, LOG_INFO);
+    ADD_INT_MACRO(module, LOG_DEBUG);
 
     /* openlog() option flags */
-    PyModule_AddIntConstant(m, "LOG_PID",         LOG_PID);
-    PyModule_AddIntConstant(m, "LOG_CONS",        LOG_CONS);
-    PyModule_AddIntConstant(m, "LOG_NDELAY",  LOG_NDELAY);
+    ADD_INT_MACRO(module, LOG_PID);
+    ADD_INT_MACRO(module, LOG_CONS);
+    ADD_INT_MACRO(module, LOG_NDELAY);
 #ifdef LOG_ODELAY
-    PyModule_AddIntConstant(m, "LOG_ODELAY",  LOG_ODELAY);
+    ADD_INT_MACRO(module, LOG_ODELAY);
 #endif
 #ifdef LOG_NOWAIT
-    PyModule_AddIntConstant(m, "LOG_NOWAIT",  LOG_NOWAIT);
+    ADD_INT_MACRO(module, LOG_NOWAIT);
 #endif
 #ifdef LOG_PERROR
-    PyModule_AddIntConstant(m, "LOG_PERROR",  LOG_PERROR);
+    ADD_INT_MACRO(module, LOG_PERROR);
 #endif
 
     /* Facilities */
-    PyModule_AddIntConstant(m, "LOG_KERN",        LOG_KERN);
-    PyModule_AddIntConstant(m, "LOG_USER",        LOG_USER);
-    PyModule_AddIntConstant(m, "LOG_MAIL",        LOG_MAIL);
-    PyModule_AddIntConstant(m, "LOG_DAEMON",  LOG_DAEMON);
-    PyModule_AddIntConstant(m, "LOG_AUTH",        LOG_AUTH);
-    PyModule_AddIntConstant(m, "LOG_LPR",         LOG_LPR);
-    PyModule_AddIntConstant(m, "LOG_LOCAL0",  LOG_LOCAL0);
-    PyModule_AddIntConstant(m, "LOG_LOCAL1",  LOG_LOCAL1);
-    PyModule_AddIntConstant(m, "LOG_LOCAL2",  LOG_LOCAL2);
-    PyModule_AddIntConstant(m, "LOG_LOCAL3",  LOG_LOCAL3);
-    PyModule_AddIntConstant(m, "LOG_LOCAL4",  LOG_LOCAL4);
-    PyModule_AddIntConstant(m, "LOG_LOCAL5",  LOG_LOCAL5);
-    PyModule_AddIntConstant(m, "LOG_LOCAL6",  LOG_LOCAL6);
-    PyModule_AddIntConstant(m, "LOG_LOCAL7",  LOG_LOCAL7);
+    ADD_INT_MACRO(module, LOG_KERN);
+    ADD_INT_MACRO(module, LOG_USER);
+    ADD_INT_MACRO(module, LOG_MAIL);
+    ADD_INT_MACRO(module, LOG_DAEMON);
+    ADD_INT_MACRO(module, LOG_AUTH);
+    ADD_INT_MACRO(module, LOG_LPR);
+    ADD_INT_MACRO(module, LOG_LOCAL0);
+    ADD_INT_MACRO(module, LOG_LOCAL1);
+    ADD_INT_MACRO(module, LOG_LOCAL2);
+    ADD_INT_MACRO(module, LOG_LOCAL3);
+    ADD_INT_MACRO(module, LOG_LOCAL4);
+    ADD_INT_MACRO(module, LOG_LOCAL5);
+    ADD_INT_MACRO(module, LOG_LOCAL6);
+    ADD_INT_MACRO(module, LOG_LOCAL7);
 
 #ifndef LOG_SYSLOG
 #define LOG_SYSLOG              LOG_DAEMON
@@ -329,14 +324,35 @@ PyInit_syslog(void)
 #define LOG_CRON                LOG_DAEMON
 #endif
 
-    PyModule_AddIntConstant(m, "LOG_SYSLOG",  LOG_SYSLOG);
-    PyModule_AddIntConstant(m, "LOG_CRON",        LOG_CRON);
-    PyModule_AddIntConstant(m, "LOG_UUCP",        LOG_UUCP);
-    PyModule_AddIntConstant(m, "LOG_NEWS",        LOG_NEWS);
+    ADD_INT_MACRO(module, LOG_SYSLOG);
+    ADD_INT_MACRO(module, LOG_CRON);
+    ADD_INT_MACRO(module, LOG_UUCP);
+    ADD_INT_MACRO(module, LOG_NEWS);
 
 #ifdef LOG_AUTHPRIV
-    PyModule_AddIntConstant(m, "LOG_AUTHPRIV",    LOG_AUTHPRIV);
+    ADD_INT_MACRO(module, LOG_AUTHPRIV);
 #endif
 
-    return m;
+    return 0;
 }
+
+static PyModuleDef_Slot syslog_slots[] = {
+    {Py_mod_exec, syslog_exec},
+    {0, NULL}
+};
+
+/* Initialization function for the module */
+
+static struct PyModuleDef syslogmodule = {
+    PyModuleDef_HEAD_INIT,
+    .m_name = "syslog",
+    .m_size = 0,
+    .m_methods = syslog_methods,
+    .m_slots = syslog_slots,
+};
+
+PyMODINIT_FUNC
+PyInit_syslog(void)
+{
+    return PyModuleDef_Init(&syslogmodule);
+}
\ No newline at end of file
